/*
 * file Stream.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import javax.wvcm.Baseline.CompareFlag;
import javax.wvcm.Baseline.CompareReport;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList.ResponseIterator;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A proxy for a stream resource.
 * 
 * A stream resource is an activity whose versions are baselines.
 * 
 * @since 1.0
 */
public interface Stream extends Activity {

    /**
     * A list of all the versions that are the current versions in this stream.
     */
    public static final PropertyName<ResourceList<Version>> VERSION_LIST =
        new PropertyName<ResourceList<Version>>("version-list"); //$NON-NLS-1$

    /**
     * Get the {@link #VERSION_LIST} property.
     * 
     * @return the {@link #VERSION_LIST} property.
     * @throws WvcmException if this Stream was not created with 
     * {@link #VERSION_LIST} as a wanted property.
     */
    public ResourceList<Version> getVersionList() throws WvcmException;

    /**
     * The stream to which changes in this stream are to be merged.
     * An implementation MAY reject changing this property once the stream
     * has been created. May be null.
     * @see #getTarget
     * @see #setTarget
     */
    public static final PropertyName<Stream> TARGET =
        new PropertyName<Stream>("target"); //$NON-NLS-1$

    /**
     * Get the {@link #TARGET} property.
     * 
     * @return the {@link #TARGET} property.
     * @throws WvcmException if this Version was not created with
     * {@link #TARGET} as a wanted property.
     */
    public Stream getTarget() throws WvcmException;

    /**
     * Set the {@link #TARGET} property.
     * 
     * @param target the {@link Stream} object that
     * identifies the {@link #TARGET} for this Stream.
     * @see #getTarget
     */
    public void setTarget(Stream target);

    /**
     * A list of all streams which are declared to merge to this stream.
     * The {@link #SOURCE_LIST} property is the computed inverse of the
     * {@link #TARGET} property.
     * @see #getSourceList
     */
    public static final PropertyName<ResourceList<Stream>> SOURCE_LIST =
        new PropertyName<ResourceList<Stream>>("source-list"); //$NON-NLS-1$

    /**
     * Get the {@link #SOURCE_LIST} property.
     * 
     * @return the {@link #SOURCE_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #SOURCE_LIST} as a wanted property.
     */
    public ResourceList<Stream> getSourceList() throws WvcmException;

    /** 
     * A workspace that can be used to browse the contents of this stream.
     * This workspace MAY refuse to allow any change other than a
     * {@link Workspace#doUpdate} whose argument is this stream.
     */
    public static final PropertyName<Workspace> WORKSPACE =
        new PropertyName<Workspace>("workspace"); //$NON-NLS-1$

    /**
     * Get the {@link #WORKSPACE} property.
     * 
     * @return the {@link #WORKSPACE} property.
     * @throws WvcmException if this Stream was not created with
     * {@link #WORKSPACE} as a wanted property.
     */
    public Workspace getWorkspace() throws WvcmException;

    /**
     * Compare this Stream with another stream.
     * 
     * @param stream the stream being compared to this Stream.
     * @param flags boolean flags for the compare report.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects, that enumerate the differences between the
     * versions selected by this Stream and the stream argument.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  This resource does not support this report.
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     * The resource identified by the <code>stream</code> argument must be a stream.
     */
    public ResponseIterator<CompareReport> doCompareReport(
            Stream stream,
            CompareFlag[] flags, 
            Feedback feedback)
    throws WvcmException;

    /**
     * Compare this Stream with a baseline.
     * 
     * @param baseline the baseline being compared to this Stream.
     * @param flags boolean flags for the compare report.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects, that enumerate the differences between the
     * versions selected by this Stream and the baseline argument.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  This resource does not support this report.
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     * The resource identified by the <code>baseline</code> argument must be a baseline.
     */
    public ResponseIterator<CompareReport> doCompareReport(
            Baseline baseline,
            CompareFlag[] flags, 
            Feedback feedback)
    throws WvcmException;

    /**
     * Update the {@link #VERSION_LIST} of this stream to contain the versions
     * specified in the sourceList argument.
     * If {@link #VERSION_LIST} already contains a version from the same VersionHistory
     * as the source version already, that version is replaced by the source version.
     * <p>
     * Postconditions:
     * <li>(update-sources): Each of the versions in the sourceList MUST have updated
     *  the {@link #VERSION_LIST} of this Stream.
     * 
     * @param sourceList the list of resources specifying the versions whose target in this stream should be updated.
     * If a member of sourceList is an activity, it must not be a stream, and
     * each version in the {@link Activity#ACTIVITY_VERSION_LIST} is a source version.
     * If a member of sourceList is a version-controlled resource,
     * the checked-in version of the member is a source version;
     * if that version-controlled resource is a folder, all members
     * of that folder are considered members of the sourceList.
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException An implementation may throw a single exception on the first error encountered,
     * or may attempt to update all of the source versions, with any errors accumulated in the
     * {@link WvcmException#getNestedExceptions} list.
     * <br>ReasonCode:
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     *  The resources identified by the <code>sourceList</code> argument must be versions, activities,
     *  streams, or controllable resources.
     * <li>{@link ReasonCode#MUST_BE_CHECKED_IN}:
     *  A version-controlled resource in the sourceList must be checked in.
     */
    public <T extends Resource> Stream doUpdate(ResourceList<T> sourceList, Feedback feedback)
    throws WvcmException;
}
